/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkToolkits.h

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef vtkToolkits_h
#define vtkToolkits_h

#pragma message                                                                                    \
  "vtkToolkits.h is deprecated. For `VTK_DEBUG_LEAKS`, check `vtkDebug.h`, for video support, see `vtkIOMovieConfigure.h` and `vtkIOVideoConfigure.h`."

#endif

// VTK-HeaderTest-Exclude: vtkToolkits.h
